print_expansion = 0.1;

base_height = 12;
base_diameter = 28;

nut_size = 16+2*print_expansion;
nut_height = 8.3+print_expansion;

knurl_radius = 2;
knurl_count = 10;

joint_size = [16, 12, 18];
joint_slot_width = 4+2*print_expansion;
joint_hole_diameter = 7+2*print_expansion;
joint_cogs_count=30;
joint_cogs_height=0.5;
joint_cogs_offset=1+2*print_expansion;

overlap=0.1;
pi=3.14;

module knurls () {
    count = knurl_count;
    height = base_height;
    radius = knurl_radius;
    surface_diameter = base_diameter;
    
    angle=360/count;
    for( number = [1 : count] ) {
        translate( [ surface_diameter*cos(number*angle)/2, surface_diameter*sin(number*angle)/2, -overlap ] )
        cylinder( $fn=24, height+2*overlap, r=radius );
    };
}

module base () {
    cylinder( $fn=base_diameter*pi*3, base_height, d=base_diameter );
}    

module nut () {
    translate( [0,0,-overlap] )
    cylinder( $fn=6, nut_height+overlap, d=(nut_size/cos(30)) );
}

module cogs() {
    count = joint_cogs_count;
    angle=360/count;

	offset = joint_cogs_offset;
	outer_radius = joint_size[0]/2-offset;
	inner_radius = joint_hole_diameter/2+offset;

    height = joint_cogs_height;
	length = outer_radius*cos(angle/2)-inner_radius;
    
	outer_width=outer_radius*sin(angle/2)*2;
	inner_width=inner_radius*tan(angle/2)*2;
	
    for( number = [1 : count] ) {
        rotate( [0,number*angle,0] )
		translate( [0,0,-outer_radius*cos(angle/2)] )
		linear_extrude( height=length, slices=ceil(length), scale=[(inner_width/outer_width),1] )
		polygon( points=[[-outer_width/2,0], [0,height], [outer_width/2,0], [outer_width/2,-overlap], [-outer_width/2,-overlap]] );
    };
}

module joint () {
    width = joint_size[0];
    depth = joint_size[1];
    height = joint_size[2];
	
	slot_width = joint_slot_width;

	union() {
		// joint without cogs
		difference() {
			// joint full body
			union() {
				// cubical part of the joint is radius less than full height
				// radius is half width of the joint
				translate( [-width/2, -depth/2, -overlap] )
				cube( [width, depth, height-width/2+overlap] );
				
				// radial part of the joint
				translate( [0,0,height-width/2] )
				rotate( [90,0,0] ) 
				cylinder( $fn=width*pi*3, depth, d=width, center=true );
			}
			
			// slot
			translate( [-width/2-overlap, -slot_width/2, -overlap*2] )
			cube( [width+overlap*2, slot_width, height+overlap*3] );
			
			// mount hole
			translate( [0,0,height-width/2] )
			rotate( [90,0,0] ) 
			cylinder( $fn=joint_hole_diameter*pi*3, depth+overlap*2, d=joint_hole_diameter, center=true );
		}
		
		// cogs
		translate( [0,-slot_width/2,height-width/2] )
		cogs();

        rotate ( [0,0,180] )
		translate( [0,-slot_width/2,height-width/2] )
		cogs();
	}
}

union() {
	difference() {
		base();
		nut ();
		knurls();
	};

	translate( [0,0,base_height] )
	joint();
}
